import time

from kivy.config import Config

Config.set('input', 'mouse', 'mouse,multitouch_on_demand')

from kivy.app import App
from kivy.uix.screenmanager import ScreenManager, Screen, FadeTransition, NoTransition
from kivy.lang.builder import Builder
from kivy.clock import Clock

from scripts import (game_manager,
                     scene,
                     input_manager,
                     graphic_data,
                     game_data,
                     story_manager,
                     dialogue_screen,
                     physics,
                     pathing,
                     gui,
                     resource_manager,
                     splash_screen,
                     stories,
                     audio_manager,
                     util)


class Application(App):
    title = 'Two Worlds'
    target_dt = 1 / 60.0
    max_dt = 1 / 20.0

    def __init__(self):
        App.__init__(self)
        Builder.load_file('kivy_scripts/prefabs.kv')

        self.audio_manager = audio_manager.AudioManager()
        self.splash_screen = splash_screen.SplashScreen(self)
        self.resource_manager = resource_manager.ResourceManager(self)
        self.pathfinder = pathing.Pathfinder()
        self.physics = physics.PhysicsManager(self)
        self.game_data = game_data.GameData()
        self.graphics = graphic_data.GraphicData()
        self.input_manager = input_manager.InputManager(self)
        self.scene = scene.Scene(self)
        self.game_manager = game_manager.GameManager(self)
        self.dialogue_screen = dialogue_screen.DialogueScreen(self)

        self.gui = gui.GUI(self)
        self.story_manager = story_manager.StoryManager(self)

    def build(self):
        Clock.schedule_interval(self.update, self.target_dt)

        # self.root = ScreenManager(transition=FadeTransition(duration=.1))
        self.root = ScreenManager(transition=NoTransition())

        game_screen = Screen(name='game')
        game_screen.add_widget(self.scene)
        game_screen.add_widget(self.gui)

        self.root.add_widget(self.splash_screen)
        self.root.add_widget(Builder.load_file('kivy_scripts/menu.kv'))
        self.root.add_widget(Builder.load_file('kivy_scripts/ending.kv'))
        self.root.add_widget(self.dialogue_screen)
        self.root.add_widget(game_screen)

        self.audio_manager.play_song('menu_song')
        self.set_screen('splash')

        return self.root

    def set_screen(self, name):
        if name == 'game':
            self.audio_manager.play_song('game_song')
        elif self.root.current == 'game':
            self.audio_manager.play_song('menu_song')
        self.root.current = name

    def start_story(self):
        self.story_manager.start_story(stories.main_story(self))

    def start_challenge(self):
        self.story_manager.start_story(stories.challenge_story(self))

    def on_esc(self):
        if self.root.current == 'game':
            self.game_manager.toggle_paused()
        elif self.root.current == 'dialogue':
            self.dialogue_screen.dialogue = None
            self.set_screen('menu')
        elif self.root.current == 'menu':
            self.stop()
        elif self.root.current == 'splash':
            self.dialogue_screen.dialogue = None
            self.set_screen('menu')

    def do_ending(self, ending):
        run_time = self.game_data.get_value('timer')

        self.dialogue_screen.dialogue = None
        ending_screen = self.root.get_screen('ending')

        items = self.game_data.get_value('items')
        if items is False:
            items = -1

        record = self.game_data.records.get(ending)
        if record is None:
            time_str = 'N/A'
            self.game_data.records[ending] = run_time
        else:
            time_str = util.get_time_str(record)
            if run_time < record:
                self.game_data.records[ending] = run_time

        ending_screen.ids['item_text'].text = '{}/9 items'.format(items + 1)
        ending_screen.ids['ending_text'].text = ending
        ending_screen.ids['time_text'].text = util.get_time_str(run_time)
        ending_screen.ids['record_text'].text = time_str
        self.set_screen('ending')

    def restart(self):
        self.set_screen('splash')

    def on_stop(self, *args):
        self.game_data.save_records()

    def update(self, dt):
        dt = min(dt, self.max_dt)
        if self.root.current == 'dialogue':
            self.game_data.data['timer'] += dt
            self.dialogue_screen.update(dt)

        elif self.root.current == 'game':
            if not self.game_manager.paused:
                self.game_data.data['timer'] += dt
                self.physics.update(dt)
                self.game_manager.update(dt)
                self.gui.update(dt)
                self.scene.update(dt)
            self.scene.update_camera(dt)
        elif self.root.current == 'splash':
            self.splash_screen.update(dt)


Application().run()
