import player, enemy, bullet, goal, item, levels, gui

PLAYING, LEAVING, FAIL = xrange(3)


class GameManager:
    def __init__(self, app):
        self.app = app

        self.player = player.Player(self.app)
        self.app.scene.add_widget(self.player)

        self.goal = goal.Goal(self.app)
        self.app.scene.add_widget(self.goal)

        self.enemies = set()
        self.bullets = set()
        self.items = set()
        self.level = None

        self.state = PLAYING
        self.state_timer = 0

        self.win_dialogue = None

        self.lose_dialogue = None

        self.paused = False

        self.time_speed = 1

    def set_state(self, state):
        if state == PLAYING:
            self.app.gui.set_state(gui.CLEAR)
            self.app.scene.warping = False

        elif state == LEAVING:
            self.state_timer = 9
            self.app.gui.set_state(gui.DISCONNECT)
            self.app.scene.warping = True
            self.app.pathfinder.set_search_area(*self.player.position.xyz[::2])
            self.refresh_enemy_paths()

        elif state == FAIL:
            self.app.gui.set_state(gui.CLEAR)
            self.state_timer = 4
            for e in self.enemies:
                if e.state != enemy.DIE:
                    e.set_state(enemy.FINISH)
            self.app.scene.warping = False
            self.goal.state = goal.FAILED

        self.state = state

    def add_bullet(self, x, y, z, angle):
        b = self.app.resource_manager.get_bullet(x, y, z, angle)
        self.bullets.add(b)
        self.app.scene.add_widget(b)

    def remove_bullet(self, b):
        if b in self.bullets:
            self.app.resource_manager.bullet_cache.add(b)
            self.bullets.remove(b)
            self.app.physics.space.remove(b.body, b.collider)
            self.app.scene.remove_widget(b)

    def add_item(self, item_name, x, y, z):
        if not self.app.game_data.get_value('got_{}'.format(item_name)):
            i = item.Item(self.app, item_name)
            i.position.xyz = x, y, z
            self.items.add(i)
            self.app.scene.add_widget(i)

    def remove_item(self, i):
        self.items.remove(i)
        self.app.scene.remove_widget(i)

    def clear_level(self):
        self.app.scene.remove_widget(self.level)
        self.level = None
        self.app.physics.clear_static()

        for e in self.enemies.copy():
            self.remove_enemy(e)

        for b in self.bullets.copy():
            self.remove_bullet(b)

        for i in self.items:
            self.app.scene.remove_widget(i)
        self.items.clear()

        self.goal.state = goal.IDLE
        self.goal.scale.x = 1
        self.goal.scale.z = 1
        self.goal.rotation_speed = 50

        self.app.scene.canvas['warp'] = 0.0
        self.app.scene.warp_speed = 0

    def add_enemy(self, node):
        e = self.app.resource_manager.get_enemy(node)
        self.enemies.add(e)
        self.app.scene.add_widget(e)

    def remove_enemy(self, e):
        if e.body in self.app.physics.space.bodies:
            self.app.physics.space.remove(e.body, e.body.shapes)
        self.app.scene.remove_widget(e)
        self.app.resource_manager.enemy_cache.add(e)
        self.enemies.remove(e)

    def refresh_enemy_paths(self):
        for e in self.enemies:
            if e.state in (enemy.PATROL, enemy.IDLE) and e.node != -1:
                if self.app.pathfinder.nodes[e.last_node][2] > self.app.pathfinder.nodes[e.node][2]:
                    e.node = e.last_node
                    e.target = self.app.pathfinder.nodes[e.node][1]
                e.set_state(enemy.CHASE)

    def toggle_paused(self):
        self.paused = not self.paused
        self.app.gui.show_paused(self.paused)

    def set_god_mode(self, value):
        if value:
            # self.player.can_damage = False
            self.time_speed = .2
            bullet.Bullet.speed = 5
        else:
            # self.player.can_damage = True
            self.time_speed = 1
            bullet.Bullet.speed = 25

    def load_level(self, level_name, win_dialogue, lose_dialogue=None):
        self.set_god_mode(False)
        self.app.dialogue_screen.dialogue = None
        self.win_dialogue = win_dialogue
        self.lose_dialogue = lose_dialogue

        self.clear_level()

        levels.LEVELS[level_name](self)

        if self.level is not None:
            self.app.scene.add_widget(self.level)

        self.player.update(0)

        self.set_state(PLAYING)
        self.app.gui.set_state(gui.BEGIN)
        self.app.set_screen('game')

    def update(self, dt):
        if not self.paused:
            self.player.update(dt)

            for e in self.enemies:
                e.update(dt, self.time_speed)
            for b in self.bullets.copy():
                b.update(dt)
            for i in self.items.copy():
                i.update(dt)

            self.goal.update(dt)

            if self.state == LEAVING:
                self.state_timer -= dt
                if self.state_timer <= 0:
                    if self.win_dialogue is not None:
                        self.app.story_manager.load_dialogue(self.win_dialogue)

            elif self.state == FAIL:
                self.state_timer -= dt

                if self.state_timer <= 0:
                    if self.lose_dialogue is not None:
                        self.app.story_manager.load_dialogue(self.lose_dialogue)
                    elif self.win_dialogue is not None:
                        self.app.story_manager.load_dialogue(self.win_dialogue)
