import math

import pymunk

from kivy.uix.widget import Widget
from kivy.graphics import *

import physics, util

WALK_ANIM, AIM_ANIM, DEATH_ANIM, IDLE_ANIM = xrange(4)

DIE = 0
ATTACK = 1


class Robot(Widget):
    max_health = 5
    draw_speed = 500
    fire_rate = .25

    friction = .92
    stop_speed = .02

    gun_offsets = (-1.3, 1), (1.3, 1)

    collision_type = physics.ENEMY

    def __init__(self, app, color):
        self.app = app

        self.can_damage = True

        self.body = pymunk.Body()
        self.body.velocity_func = self.velocity_func
        self.body.parent = self
        self.collider = pymunk.Circle(self.body, 1)
        self.collider.mass = 1
        self.collider.collision_type = self.collision_type
        if self.collision_type == physics.ENEMY:
            self.collider.filter = physics.ENEMY_FILTER
        else:
            self.collider.filter = physics.PLAYER_FILTER

        self.app.physics.space.add(self.body, self.collider)

        Widget.__init__(self)
        with self.canvas:
            PushMatrix()

            self.position = Translate()
            self.rotation = Rotate(0, 0, 1, 0)
            self.color = Color(*color)
            self.app.graphics.draw_scene('models/robot_body')

            PushMatrix()
            Translate(0, 3.75, 0)
            self.head_yaw = Rotate(0, 0, 1, 0)
            self.head_pitch = Rotate(0, 1, 0, 0)
            self.app.graphics.draw_scene('models/robot_head')
            PopMatrix()

            PushMatrix()
            Translate(0, 3.25, 1.25)
            self.right_arm_rot = Rotate(30, 0, 0, 1)
            self.app.graphics.draw_scene('models/robot_arm')
            PopMatrix()

            PushMatrix()
            Translate(0, 3.25, -1.25)
            self.left_arm_rot = Rotate(30, 0, 0, 1)
            self.app.graphics.draw_scene('models/robot_arm')
            PopMatrix()

            PushMatrix()
            Translate(0, 1.75, .4)
            self.right_leg_rot = Rotate(30, 0, 0, 1)
            self.app.graphics.draw_scene('models/robot_leg')
            PopMatrix()

            PushMatrix()
            Translate(0, 1.75, -.4)
            self.left_leg_rot = Rotate(30, 0, 0, 1)
            self.app.graphics.draw_scene('models/robot_leg')
            PopMatrix()

            PopMatrix()

        self.health = self.max_health

        self.state = -1
        self.state_timer = 0

        self.anim = IDLE_ANIM
        self.anim_time = 0

        self.target_angle = 0

    def velocity_func(self, *args):
        self.body.velocity *= self.friction
        if self.body.velocity.get_length_sqrd() < self.stop_speed:
            self.body.velocity *= 0

    def set_state(self, state):
        self.state = state

    def set_position(self, x=0, y=0, z=0):
        self.body.position = (x, z)
        self.position.y = y

    def face_velocity(self, dt):
        self.target_angle = -self.body.velocity.get_angle() * 57.3

    def damage(self, amt):
        if self.health > 0 and self.can_damage:
            self.health -= amt
            if self.health <= 0:
                self.set_state(DIE)
                self.set_animation(DEATH_ANIM)
            else:
                self.on_damage()

    def on_damage(self):
        pass

    def fire(self, index):
        r = self.target_angle / 57.3

        cos = math.cos(r)
        sin = math.sin(r)

        iy, ix = self.gun_offsets[index]

        fx = ix * cos + iy * sin
        fy = ix * -sin + iy * cos

        self.app.game_manager.add_bullet(self.position.x + fx, 3.25, self.position.z + fy, -self.target_angle)

    def set_animation(self, anim):
        if anim == DEATH_ANIM:
            self.anim_time = 0
        self.anim = anim

    def do_animation(self, dt, factor=1):
        if self.anim == DEATH_ANIM:
            self.anim_time += dt
            self.head_yaw.angle = util.loop_time(self.anim_time, 2) * 90 - 135
            self.head_pitch.angle = -10

            if self.anim_time < 1:
                self.position.y = self.anim_time * -1.5
                self.left_leg_rot.angle = self.anim_time * 90
                self.right_leg_rot.angle = self.anim_time * 90
                self.left_arm_rot.angle = (1 - self.anim_time) * 90
                self.right_arm_rot.angle = (1 - self.anim_time) * 90
        elif self.anim == IDLE_ANIM:
            self.anim_time += dt
            f = 1 - (5 * dt)
            self.head_pitch.angle = util.loop_time(self.anim_time, 2) * 30
            self.head_yaw.angle = (self.head_yaw.angle - 90) * f + 90
            self.position.y *= f
            self.left_leg_rot.angle *= f
            self.right_leg_rot.angle *= f
            self.left_arm_rot.angle *= f
            self.right_arm_rot.angle *= f
        else:
            value = util.loop_time(self.anim_time, 7) * 2 - 1
            self.left_leg_rot.angle = value * 45
            self.right_leg_rot.angle = value * -45
            self.position.y = (1 - abs(value)) * .1

            if self.anim == WALK_ANIM:
                self.left_arm_rot.angle = value * -45
                self.right_arm_rot.angle = value * 45
            elif self.anim == AIM_ANIM:
                if self.left_arm_rot.angle < 90:
                    self.left_arm_rot.angle += self.draw_speed * dt * factor
                    if self.left_arm_rot.angle > 90:
                        self.left_arm_rot.angle = 90

                if self.right_arm_rot.angle < 90:
                    self.right_arm_rot.angle += self.draw_speed * dt * factor
                    if self.right_arm_rot.angle > 90:
                        self.right_arm_rot.angle = 90

    def update_shooting(self, dt):
        arm_index = int(self.state_timer > self.fire_rate)
        self.state_timer -= dt
        if self.state_timer < arm_index * self.fire_rate:
            self.fire(arm_index)
            if arm_index is 0:
                self.state_timer += self.fire_rate * 2

    def update(self, dt, factor=1):
        if self.state != DIE:
            self.position.x = self.body.position.x
            self.position.z = self.body.position.y

            speed = self.body.velocity[0] ** 2 + self.body.velocity[1] ** 2

            if speed > self.stop_speed:
                if self.anim not in (AIM_ANIM, WALK_ANIM):
                    self.set_animation(WALK_ANIM)
                self.anim_time += dt * speed ** .5

            elif self.anim == WALK_ANIM:
                self.set_animation(IDLE_ANIM)

            if self.target_angle:
                angle_diff = util.angle_diff(self.rotation.angle, self.target_angle)
                self.rotation.angle += angle_diff * 10 * dt * factor

